"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require("lodash");

var _utils = require("../../utils");

var ruleName = "indentation";
exports.ruleName = ruleName;
var messages = (0, _utils.ruleMessages)(ruleName, {
  expected: function expected(x) {
    return "Expected indentation of " + x;
  }
});

exports.messages = messages;
/**
 * @param {number|"tab"} space - Number of whitespaces to expect, or else
 *   keyword "tab" for single `\t`
 * @param {object} options
 * @param {array} except = ["block", "value"] - Whether extra level of
 *   indentation should be used for nested blocks and multi-line values respectively
 */

exports["default"] = function (space, options) {
  var isTab = space === "tab";

  var indentChar = isTab ? "\t" : (0, _lodash.repeat)(" ", space);

  var warningWord = isTab ? "tab" : "space";

  function legibleExpectation(level, line) {
    var count = isTab ? level : level * space;
    var quantifiedWarningWord = count === 1 ? warningWord : warningWord + "s";
    return count + " " + quantifiedWarningWord + " at line " + line;
  }

  return function (root, result) {
    root.eachRule(checkNode);
    root.eachAtRule(checkNode);
    root.eachDecl(checkNode);

    function checkNode(node) {

      // Expected whitespace equals the indent character
      // repated according to the indentation level
      var nodeLevel = indentationLevel(node);

      // unless option.block is "never", in which case
      // everything is taken down a level (all blocks
      // are level 0)
      if ((0, _utils.optionsHaveException)(options, "block") && nodeLevel > 0) {
        nodeLevel--;
      }

      var expectedWhitespace = (0, _lodash.repeat)(indentChar, nodeLevel);

      var before = node.before;
      var after = node.after;

      // Only inspect the spaces before the node
      // if this is the first node in root
      // or there is a newline in the `before` string.
      // (If there are no lines between two statements,
      // we don't want to check for indentation.)
      var inspectBefore = isFirstNodeInRoot(node) || before.indexOf("\n") !== -1;

      // Inspect whitespace in the `before` string that is
      // *after* the *last* newline character,
      // because anything besides that is not indentation for this node:
      // it is separation, checked by a separate rule
      if (inspectBefore && before.slice(before.lastIndexOf("\n") + 1) !== expectedWhitespace) {
        (0, _utils.report)({
          message: messages.expected(legibleExpectation(nodeLevel, node.source.start.line)),
          node: node,
          line: node.source.start.line,
          result: result,
          ruleName: ruleName
        });
      }

      // Only blocks have the `after` string to check.
      // Only inspect `after` strings that start with a newline,
      // otherwise there's no indentation involved
      if (after && after.indexOf("\n") !== -1 && after.slice(after.lastIndexOf("\n") + 1) !== expectedWhitespace) {
        (0, _utils.report)({
          message: messages.expected(legibleExpectation(nodeLevel, node.source.end.line)),
          node: node,
          line: node.source.end.line,
          result: result,
          ruleName: ruleName
        });
      }

      // If this is a declaration, check the value
      if (node.value) {
        checkValue(node, nodeLevel);
      }
    }

    function isFirstNodeInRoot(node) {
      return root.first === node;
    }

    function checkValue(node, declLevel) {
      var value = node.value;
      if (value.indexOf("\n") === -1) {
        return;
      }

      var valueLevel = (0, _utils.optionsHaveException)(options, "value") ? declLevel : declLevel + 1;
      var postNewlineExpected = (0, _lodash.repeat)(indentChar, valueLevel);

      (0, _utils.styleSearch)({ source: value, target: "\n" }, function (match, newlineCount) {
        // Starting at the index after the newline, we want to
        // check that the whitespace characters before the first
        // non-whitespace character equal the expected indentation
        var postNewlineActual = /^(\s*)\S/.exec(value.slice(match.startIndex + 1))[1];

        if (postNewlineActual !== postNewlineExpected) {

          var line = node.source.start.line + newlineCount;

          (0, _utils.report)({
            message: messages.expected(legibleExpectation(valueLevel, line)),
            node: node,
            line: line,
            result: result,
            ruleName: ruleName
          });
        }
      });
    }
  };
};

// Indentation level equals the ancestor nodes
// separating this node from root
function indentationLevel(_x2) {
  var _arguments = arguments;
  var _again = true;

  _function: while (_again) {
    var node = _x2;
    level = undefined;
    _again = false;
    var level = _arguments.length <= 1 || _arguments[1] === undefined ? 0 : _arguments[1];

    if (node.parent.type !== "root") {
      _arguments = [_x2 = node.parent, level + 1];
      _again = true;
      continue _function;
    }
    return level;
  }
}